#!/usr/bin/env python3
"""
Package entry point.
"""
import sys

from auto_tag import entrypoint

def main():
    """Main entry point for module and cli."""
    entrypoint.main(sys.argv[1:])


if __name__ == '__main__':
    main()



#!/usr/bin/env python3
"""
Automatically tags branches base on commit message
"""
import logging

import semantic_version
import git

from auto_tag import constants
from auto_tag import git_custom_env
from auto_tag import tag_search_strategy


class AutoTag():
    """Class  wrapper for auto-tag functionality."""

    def __init__(self, repo, branch, upstream_remotes, detectors,
                 search_strategy=tag_search_strategy.DEFAULT_STRATEGY,
                 git_name=None, git_email=None, logger=None,
                 append_v=False, skip_if_exists=False):
        """Initializa the AutoTag class.

        :param logger: If an existing logger is to be used
        :param args: CLI arguments
        """
        self._logger = logger or logging.getLogger(__name__)
        self._repo = repo
        self._detectors = detectors
        self._branch = branch
        self._upstream_remotes = upstream_remotes or []
        self._search_strategy = search_strategy
        self._git_name = git_name
        self._git_email = git_email
        self._append_v = append_v

        self._skip_if_exists = skip_if_exists

    def get_latest_tag(self, repo):
        """Return the last tag for the given repo in a Version class.

        :param repo: Repository to query for tags
        :type repo: git.Repo

        :returns: The latest tag from the repository
        :rtype: (git.Tag, semantic_version.Version)
        """
        raw_tag = self._search_strategy(
            repo=repo, branch=self._branch)
        if raw_tag is None:
            return None, None
        sem_tag = semantic_version.Version(
            tag_search_strategy.clean_tag_name(str(raw_tag)))
        return raw_tag, sem_tag

    @staticmethod
    def bump_tag(tag, change_type):
        """Return the next tag version based on the change type."""
        if tag is None:
            return semantic_version.Version('0.0.1')

        if change_type == constants.MAJOR:
            return tag.next_major()

        if change_type == constants.MINOR:
            return tag.next_minor()

        return tag.next_patch()

    def get_all_commits_from_a_tag(self, repo, branch, tag):
        """
        Return all commits from the branch that
        happened father the specified tag.

        :param repo: Repository to query for commits
        :type repo: git.Repo

        :param branch: Branch to work on
        :type branch: str

        :param repo: Tag to stop the query at.
        :type repo: semantic_version.Version

        :returns: List of commits.
        :rtype: list of git.Commit
        """
        stop_commit = None
        if str(tag) in repo.tags:
            stop_commit = repo.tags[str(tag)].commit

        commits = []
        for commit in repo.iter_commits(rev=branch):
            if commit == stop_commit:
                break
            commits.append(commit)
        self._logger.debug(
            'Commits found from after tag %s: %s', tag, commits)
        return commits

    def get_change_type(self, commits):
        """Evaluate all detectors on a commit and decide on the change type."""
        change_type = constants.PATCH
        for commit in commits:
            for detector in self._detectors:
                if detector.evaluate(commit):
                    change_type = max(change_type, detector.change_type)

        return change_type

    @staticmethod
    def get_remote(repo, name):
        """Return the git.Remote object base on the name."""
        for remote in repo.remotes:
            if remote.name == name:
                return remote
        return None

    def push_to_remotes(self, repo, tag):
        """Push a tag to the specified remotes."""
        if self._upstream_remotes:
            self._logger.info('Start pushing to remotes: %s.',
                              self._upstream_remotes)
        else:
            self._logger.info('No push remote was specified')
            return
        for remote_name in self._upstream_remotes:
            remote = self.get_remote(repo, remote_name)
            if remote:
                self._logger.info('Push %s to %s', tag, remote)
                remote.push(str(tag))
            else:
                self._logger.error(
                    'Can\'t find remote with name `%s`', remote_name)

    @staticmethod
    def _create_tag_message(commits, tag):
        """Create a tag message that contains informations
        from the commits. """

        tag_message = 'Release {} \n\n'.format(str(tag))

        for message in [c.message for c in commits]:
            tag_message += '    * {}\n'.format(message.split('\n')[0].strip())
        return tag_message

    @staticmethod
    def _is_last_commit_already_tagged(repo, last_tag, branch_name):
        """Check if the last_tag is also applied on the latest commit."""
        if last_tag is None:
            return False
        commit = list(repo.iter_commits(rev=branch_name))[0]
        return last_tag.commit == commit

    def work(self):
        """Main entry point.

        :param args: Argument to work on
        """
        repo = git.Repo(self._repo, odbt=git.GitDB)
        self._logger.info('Start tagging %s', repo)
        last_tag, latest_tag_sem = self.get_latest_tag(repo)

        self._logger.info('Found tag %s', last_tag)
        commits = self.get_all_commits_from_a_tag(
            repo, self._branch, last_tag)
        type_of_change = self.get_change_type(commits)
        next_tag = self.bump_tag(latest_tag_sem, type_of_change)
        # NOTE(mmicu): Here we need to check if the next tag exists
        tag = 'v{}'.format(next_tag) if self._append_v else str(next_tag)

        self._logger.info('Bumping tag %s -> %s', last_tag, next_tag)

        with git_custom_env.GitCustomeEnvironment(repo.working_dir,
                                                  self._git_name,
                                                  self._git_email):
            tag_on_last_commit = self._is_last_commit_already_tagged(
                repo, last_tag, self._branch)

            if self._skip_if_exists and tag_on_last_commit:
                self._logger.info(
                    ('The tag is already tagged, following your CLI option'
                     ' we will skip tagging.'))
            else:
                repo.create_tag(
                    str(tag),
                    message=self._create_tag_message(commits, next_tag))

        self.push_to_remotes(repo, tag)



#!/usr/bin/env python3
"""
Constants used in the AutoTag project
"""
MAJOR = 100
MINOR = 10
PATCH = 1

CHANGE_TYPE_PAIRS = (
    (MAJOR, 'MAJOR'),
    (MINOR, 'MINOR'),
    (PATCH, 'PATCH'),
)

SEARCH_STRATEGY_BIGGEST_TAG_IN_REPO = 'biggest-tag-in-repo'
SEARCH_STRATEGY_BIGGEST_TAG_IN_BRANCH = 'biggest-tag-in-branch'
SEARCH_STRATEGY_LATEST_TAG_IN_REPO = 'latest-tag-in-repo'
SEARCH_STRATEGY_LATEST_TAG_IN_BRANCH = 'latest-tag-in-branch'

SEARCH_STRATEGYS = [
    SEARCH_STRATEGY_BIGGEST_TAG_IN_REPO,
    SEARCH_STRATEGY_BIGGEST_TAG_IN_BRANCH,
    SEARCH_STRATEGY_LATEST_TAG_IN_REPO,
    SEARCH_STRATEGY_LATEST_TAG_IN_BRANCH,
]

# pylint: disable=unnecessary-comprehension
CHANGE_TYPES = {value: name for value, name in CHANGE_TYPE_PAIRS}
CHANGE_TYPES_REVERSE = {name: value for value, name in CHANGE_TYPE_PAIRS}

PREFIX_TO_ELIMINATE = ['v']

DEFAULT_CONFIG_DETECTORS = """
detectors:

  check_for_feature_heading:
    type: CommitMessageHeadStartsWithDetector
    produce_type_change: MINOR
    params:
      pattern: 'feature'


  check_for_breaking_change:
    type: CommitMessageContainsDetector
    produce_type_change: MAJOR
    params:
      pattern: 'BREAKING_CHANGE'
      case_sensitive: false
"""



#!/usr/bin/env python3
"""
Package entry point.
"""
import sys
import logging
import logging.config

from auto_tag import core, cli, detectors_config, tag_search_strategy


def main(cli_args):
    """Main entry point for Auto-Tag module."""

    parser = cli.get_parser()
    args = parser.parse_args(cli_args)

    logger = logging.getLogger(__name__)
    # pylint:disable=no-member, protected-access
    logger.setLevel(logging._nameToLevel[args.logging])
    console_handler = logging.StreamHandler()
    console_handler.setLevel(logging.DEBUG)

    # create formatter
    formatter = logging.Formatter(
        '%(asctime)s: %(message)s')

    # add formatter to console_handler
    console_handler.setFormatter(formatter)

    # add console_handler to logger
    logger.addHandler(console_handler)

    if args.config:
        config = detectors_config.DetectorsConfig.from_file(
            args.config)
    else:
        config = detectors_config.DetectorsConfig.from_default()

    search_strategy = tag_search_strategy.SEARCH_METHODS_MAPPING[
        args.tag_search_strategy]
    autotag = core.AutoTag(
        repo=args.repo, branch=args.branch,
        upstream_remotes=args.upstream_remote,
        detectors=config.detectors,
        search_strategy=search_strategy,
        git_name=args.name, git_email=args.email,
        append_v=args.append_v_to_tag,
        skip_if_exists=args.skip_tag_if_one_already_present,
        logger=logger
    )
    autotag.work()


if __name__ == '__main__':
    main(sys.argv[1:])



#!/usr/bin/env python3
"""
Prepare Detectors based on a file config.
"""
import logging
import io
import yaml


from auto_tag import exception
from auto_tag import detectors
from auto_tag import constants

# pylint:disable=too-few-public-methods


class DetectorsConfig():
    """Handles instantiation of detectors based on a config file."""

    def __init__(self, data, logger=None):
        """Initiate the config."""
        self._data = data
        self._logger = logger or logging.getLogger(__name__)
        self._detectors = None

    @classmethod
    def from_file(cls, filepath, logger=None):
        """Read config from a file."""
        with open(filepath, 'r') as file_stream:
            return cls(file_stream.read(), logger)

    @classmethod
    def from_default(cls, logger=None):
        """Return a configuration of the default setting."""
        return cls(constants.DEFAULT_CONFIG_DETECTORS, logger)

    def _parse_detectors(self):
        """Parse the file and instantiate detectors."""
        self._detectors = []
        stream = io.StringIO(self._data)
        try:
            data = yaml.safe_load(stream)
        except yaml.YAMLError as exc:
            raise exception.BaseAutoTagException(
                'Can\'t handle config {}'.format(exc))

        if 'detectors' not in data:
            raise exception.ConfigurationError("Can't find key detectors")

        for detector_name, data in data['detectors'].items():
            self._logger.debug('Found detector %s', detector_name)
            if 'type' not in data:
                raise exception.ConfigurationError(
                    'Can\'t find type for detector {}'.format(detector_name))
            detector_class = detectors.detector_factory(data['type'])

            self._logger.debug('Detector %s is %s',
                               detector_name, detector_class)

            if 'produce_type_change' not in data:
                raise exception.ConfigurationError(
                    'Can\'t find produce_type_change for detector {}'.format(
                        detector_name))
            change_type = data['produce_type_change']
            self._logger.debug('Detector %s produces %s',
                               detector_name, change_type)

            extra_args = data.get('params', {})
            self._logger.debug('Detector %s has extra args %s',
                               detector_name, extra_args)
            detector_obj = detector_class(
                name=detector_name, change_type=change_type, **extra_args)

            self._logger.info('Prepared detector %s of type %s -> %s',
                              detector_name, detector_class, change_type)
            self._detectors.append(detector_obj)

    @property
    def detectors(self):
        """Return list of detectors."""
        if self._detectors is None:
            self._parse_detectors()
        return self._detectors



#!/usr/bin/env python3
"""
Automatically tags branches base on commit message
"""
import git


class GitCustomeEnvironment():
    """Custom Git Configuration context manager."""

    # pylint: disable=no-member
    def __init__(self, repo_path, name, email):
        """Initialize the context manager."""
        self._repo = git.Repo(repo_path, odbt=git.GitDB)
        self._name = name
        self._email = email
        self._old_name = None
        self._old_email = None
        with self._repo.config_writer() as confi_w:
            if confi_w.has_option('user', 'name'):
                self._old_name = confi_w.get_value('user', 'name')

            if confi_w.has_option('user', 'email'):
                self._old_email = confi_w.get_value('user', 'email')

    def __enter__(self):
        with self._repo.config_writer() as config_writer:
            if self._name is not None:
                config_writer.set_value('user', 'name', self._name)
            if self._email is not None:
                config_writer.set_value('user', 'email', self._email)

    def __exit__(self, _type, value, traceback):
        with self._repo.config_writer() as conf_w:
            if not conf_w.has_section('user'):
                return
            if self._old_name is None:
                conf_w.remove_option('user', 'name')
            else:
                conf_w.set_value('user', 'name', self._old_name)

            if self._old_email is None:
                conf_w.remove_option('user', 'email')
            else:
                conf_w.set_value('user', 'email', self._old_email)



#!/usr/bin/env python3
"""
Automatically tags branches base on commit message
"""
import time

import semantic_version

from auto_tag import constants
from auto_tag import exception


# pylint: disable=unused-argument
def clean_tag_name(tag_name):
    """Remove common mistakes when using semantic versioning."""
    for prefix in constants.PREFIX_TO_ELIMINATE:
        if tag_name.startswith(prefix):
            clean_tag = tag_name[len(prefix):]
            return clean_tag
    return tag_name


def get_biggest_tag_in_repo(repo, *args, **kwargs):
    """Return the last tag for the given repo in a Version class.
    :param repo: Repository to query for tags
    :type repo: git.Repo

    :returns: The latest tag from the repository.
    :rtype: str
    """
    sem_versions = [
        (
            tag,
            semantic_version.Version(clean_tag_name(tag.name))
        ) for tag in repo.tags
    ]

    if sem_versions:
        latest_tag, _ = max(sem_versions, key=lambda x: x[1])
        return latest_tag
    return None


def _get_tags_on_branch(repo, branch_name):
    """Get all the tags on this specific branch.

    :param repo: Repository to query for tags
    :type repo: git.Repo

    :param branch_name: name of the branch
    :type branch_name: string

    :returns: List of tags from this branch
    :rtype: list
    """
    commits_to_tag = {tag.commit: tag for tag in repo.tags}
    found_tags = []

    for commit in repo.iter_commits(rev=branch_name):
        if commit in commits_to_tag:
            found_tags.append(
                commits_to_tag[commit])
    return found_tags


def get_biggest_tag_in_branch(repo, branch, *args, **kwargs):
    """Return the last tag for the given repo in a Version class.
    :param repo: Repository to query for tags
    :type repo: git.Repo

    :returns: The latest tag from the repository
    :rtype: str
    """
    tags = _get_tags_on_branch(repo, branch)

    sem_versions = [
        (
            tag,
            semantic_version.Version(clean_tag_name(tag.name))
        ) for tag in tags
    ]
    if sem_versions:
        latest_tag, _ = max(sem_versions, key=lambda x: x[1])
        return latest_tag
    return None


def get_latest_tag_in_repo(repo, branch, *args, **kwargs):
    """Return the last tag for the given repo in a Version class.
    :param repo: Repository to query for tags
    :type repo: git.Repo

    :returns: The latest tag from the repository
    :rtype: str
    """
    committed_date_to_tag = [
        (time.gmtime(tag.commit.committed_date), tag) for tag in repo.tags
    ]
    # if there are no tags
    if not committed_date_to_tag:
        return None

    committed_date_to_tag.sort(key=lambda x: x[0], reverse=True)

    return committed_date_to_tag[0][1]


def get_latest_tag_in_branch(repo, branch, *args, **kwargs):
    """Return the last tag for the given repo in a Version class.
    :param repo: Repository to query for tags
    :type repo: git.Repo

    :returns: The latest tag from the repository
    :rtype: str
    """
    committed_date_to_tag = [
        (
            time.gmtime(tag.commit.committed_date),
            tag
        ) for tag in _get_tags_on_branch(repo, branch)
    ]
    # if there are no tags
    if not committed_date_to_tag:
        return None

    committed_date_to_tag.sort(key=lambda x: x[0], reverse=True)

    return committed_date_to_tag[0][1]


SEARCH_METHODS_MAPPING = {
    constants.SEARCH_STRATEGY_BIGGEST_TAG_IN_REPO:
        get_biggest_tag_in_repo,

    constants.SEARCH_STRATEGY_BIGGEST_TAG_IN_BRANCH:
        get_biggest_tag_in_branch,

    constants.SEARCH_STRATEGY_LATEST_TAG_IN_REPO:
        get_latest_tag_in_repo,

    constants.SEARCH_STRATEGY_LATEST_TAG_IN_BRANCH:
        get_latest_tag_in_branch
}

DEFAULT_STRAGETY_NAME = constants.SEARCH_STRATEGY_BIGGEST_TAG_IN_BRANCH
DEFAULT_STRATEGY = SEARCH_METHODS_MAPPING[DEFAULT_STRAGETY_NAME]


def get_last_tag(repo, branch, search_method_name):
    """Get the last tag according to the appropriate search strategy."""
    search_strategy = SEARCH_METHODS_MAPPING.get(search_method_name, None)

    if search_strategy is None:
        raise exception.UnknowkSearchStrategy(
            '{} is not a search strategy supported.Choose form {}'.format(
                search_method_name, SEARCH_METHODS_MAPPING.keys()))



#!/usr/bin/env python3
"""
Change type detectors.

Parses a configuration with what type of change do you want to produce.
"""
import abc
import logging
import re
import six

from auto_tag import constants
from auto_tag import exception


@six.add_metaclass(abc.ABCMeta)
class BaseDetector():
    """Base detector class."""

    def __init__(self, name, change_type, strip=True, **kwargs):
        """Initialize the detector."

        :param name: Name of the detector
        :param change_type: If this detector fires what type of change it will
                            create
        :param strip: Strip the evaluated value before processing
        :param logger: If specified what logger to use
        """
        self._name = name
        self._change_type_name = change_type
        self._strip = strip
        self._logger = (
            kwargs.get('logger', None) or logging.getLogger(__name__))

    @property
    def name(self):
        """Return name of the detector."""
        return self._name

    @property
    def strip(self):
        """Return strip value of the detector."""
        return self._strip

    @property
    def change_type_name(self):
        """Return the type of change this detector imposes."""
        return self._change_type_name

    @property
    def change_type(self):
        """Return the type of change this detector imposes."""
        return constants.CHANGE_TYPES_REVERSE[self._change_type_name]

    def validate_detector_params(self):
        """Check if all the parameters given to the detector make sens."""
        if self._change_type_name not in constants.CHANGE_TYPES.values():
            raise exception.DetectorValidationException(
                ('Change type {} is not in not valid. Accepted'
                 ' change types are {}').format(
                     self._change_type_name, constants.CHANGE_TYPES.keys()))

    @abc.abstractmethod
    def evaluate(self, commit):
        """Evaluate the commit and see if this detector is triggered.

        :param commit: The commit to evaluate

        :returns: True if this detector got triggered
        :rtype: book
        """


class BasePatternBaseDetector(BaseDetector):
    """Check if the commit message respects a pattern"""

    def __init__(self, *args, **kwargs):
        """Initialize the detector.

        :param pattern: The pattern to match if it starts with

        :param *args: Check with the base class
        :param **kwargs: Check with the base class
        """
        super(BasePatternBaseDetector, self).__init__(
            *args, **kwargs)

        self._pattern = kwargs.get('pattern', None)

    @property
    def pattern(self):
        """Return pattern value of the detector."""
        return self._pattern

    def validate_detector_params(self):
        """Check if all the parameters given to the detector make sens."""
        super(BasePatternBaseDetector, self).validate_detector_params()
        if not isinstance(self._pattern, str):
            raise exception.DetectorValidationException(
                ('Patter: {} is not valid.'
                 'it must be specified and of type string').format(
                     self._pattern))


class BasePatternSimpleComparationDetector(BasePatternBaseDetector):
    """Check if the commit message respects a pattern"""

    def __init__(self, *args, **kwargs):
        """Initialize the detector.

        :param pattern: The pattern to match if it starts with

        :param *args: Check with the base class
        :param **kwargs: Check with the base class
        """
        super(BasePatternSimpleComparationDetector, self).__init__(
            *args, **kwargs)

        self._case_sensitive = kwargs.get('case_sensitive', True)

    @property
    def case_sensitive(self):
        """Return case_sensitive value of the detector."""
        return self._case_sensitive

    def validate_detector_params(self):
        """Check if all the parameters given to the detector make sens."""
        super(
            BasePatternSimpleComparationDetector,
            self).validate_detector_params()
        if not isinstance(self._case_sensitive, bool):
            raise exception.DetectorValidationException(
                ('case_sensitive: {} is not valid.'
                 'it must be of type bool').format(
                     self._case_sensitive))

    def __prepare_text(self, text):
        """Prepare a text according to the config."""
        if not self._case_sensitive:
            text = text.lower()
        if self._strip:
            text = text.strip()
        return text

    def _prepare_commit_message(self, commit):
        """Get the prepared commit message according to the config."""
        return self.__prepare_text(commit.message)


class CommitMessageHeadStartsWithDetector(
        BasePatternSimpleComparationDetector):
    """Check if the head of the commit message has a particular pattern."""

    def evaluate(self, commit):
        """Check if the commit message head starts with a given string

        :param commit: The commit to evaluate

        :returns: True if this detector got triggered
        :rtype: book
        """
        message = self._prepare_commit_message(commit)

        if self._case_sensitive:
            return message.startswith(self._pattern)
        return message.startswith(self._pattern.lower())


class CommitMessageContainsDetector(BasePatternSimpleComparationDetector):
    """Check if the message of the commit contains a particular pattern."""

    def evaluate(self, commit):
        """Check if the commit message contains a given string

        :param commit: The commit to evaluate

        :returns: True if this detector got triggered
        :rtype: book
        """
        message = self._prepare_commit_message(commit)

        if self._case_sensitive:
            return self._pattern in message
        return self._pattern.lower() in message


class CommitMessageMatchesRegexDetector(BasePatternBaseDetector):
    """Check if the message of the commit matches regex pattern."""

    def __init__(self, *args, **kwargs):
        """Initialize the detector.

        :param pattern: The pattern to match if it starts with

        :param *args: Check with the base class
        :param **kwargs: Check with the base class
        """
        super(CommitMessageMatchesRegexDetector, self).__init__(
            *args, **kwargs)

        self._compiled_regex = re.compile(self._pattern)

    def validate_detector_params(self):
        """Check if all the parameters given to the detector make sens."""
        super(
            CommitMessageMatchesRegexDetector, self).validate_detector_params()

        if self. _compiled_regex is None:
            raise exception.DetectorValidationException(
                ('Patter: {} is not valid regex.'
                 'it must be specified and compliant').format(self._pattern))

    def evaluate(self, commit):
        """Check if the commit message matches a regex pattern

        :param commit: The commit to evaluate

        :returns: True if this detector got triggered
        :rtype: book
        """
        return bool(self._compiled_regex.search(commit.message))


DETECTORS = [
    CommitMessageHeadStartsWithDetector,
    CommitMessageContainsDetector,
    CommitMessageMatchesRegexDetector,
]


def detector_factory(detector_name):
    """Return the appropriate detector class based on the name.

    :param detector_name: Name of the detector
    :return: Detector class
    :rtype: auto_tag.detectors.BaseDetector
    """
    detector_map = {detector.__name__: detector for detector in DETECTORS}

    if detector_name not in detector_map:
        raise exception.DetectorNotFound(
            'Detector {} not found. Available detectors: {}'.format(
                detector_name, detector_map.keys()))
    return detector_map[detector_name]



#!/usr/bin/env python3
"""
Exception used in the AutoTag project
"""


class BaseAutoTagException(Exception):
    """Base exception for the AutoTag project."""


class DetectorValidationException(BaseAutoTagException):
    """Validation failed on a detector"""


class DetectorNotFound(BaseAutoTagException):
    """Validation failed on a detector"""


class ConfigurationError(BaseAutoTagException):
    """Validation failed on a detector"""


class CantFindBranch(BaseAutoTagException):
    """Can't find a specific branch"""


class UnknowkSearchStrategy(BaseAutoTagException):
    """Invalid search strategy."""



#!/usr/bin/env python3
"""
CLI parser for auto-tag.
"""
import logging
import argparse

from auto_tag import constants
from auto_tag import tag_search_strategy


def get_parser():
    """Return the argument parser setup."""
    parser = argparse.ArgumentParser(
        description='Tag branch based on commit messages')
    parser.add_argument('-b', '--branch', type=str, default='master',
                        help='On what branch to work on. Default `master`')
    parser.add_argument('-r', '--repo', type=str, default='.',
                        help='Path to repository. Default `.`')
    parser.add_argument('-u', '--upstream_remote', type=str, nargs='*',
                        help=('To what remote to push to.'
                              'Can be specified multiple time.'))
    #  pylint:disable=no-member, protected-access
    parser.add_argument('-l', '--logging', type=str, default='INFO',
                        help='Logging level.',
                        choices=list(logging._nameToLevel.keys()))

    parser.add_argument('--name', type=str, default=None,
                        help=('User name used for creating git objects.'
                              'If not specified the system one will be used.'))
    parser.add_argument('--email', type=str, default=None,
                        help=('Email name used for creating git objects.'
                              'If not specified the system one will be used.'))

    parser.add_argument('-c', '--config', type=str, default=None,
                        help='Path to detectors configuration.')

    parser.add_argument('--skip-tag-if-one-already-present',
                        action='store_true',
                        help=('If a tag is already present on the latest '
                              'commit don\'t apply a new tag'))

    parser.add_argument('--append-v-to-tag', action='store_true',
                        help='Append a v to the tag (ex v1.0.5)')

    parser.add_argument('--tag-search-strategy',
                        choices=constants.SEARCH_STRATEGYS,
                        default=tag_search_strategy.DEFAULT_STRAGETY_NAME,
                        help='Strategy for searching the tag.')

    return parser



#!/usr/bin/env python3
"""
Setup file for the package.
"""

import os
import pathlib
from setuptools import setup, find_packages

# The directory containing this file
HERE = os.path.abspath(pathlib.Path(__file__).parent)

# The text of the README file
README = open(
    os.path.join(HERE, 'README.md'),
    'r').read()

# This call to setup() does all the work
setup(
    name='auto-tag',
    version='0.8.0',
    description='Automatically tag a branch based on commit message',
    long_description=README,
    long_description_content_type='text/markdown',
    author='Matei-Marius Micu',
    author_email='contact@mateimicu.com	',
    license='MIT',
    url='https://github.com/mateimicu/auto-tag',
    classifiers=[
        'License :: OSI Approved :: MIT License',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.7',
        'Development Status :: 3 - Alpha',
        'Environment :: MacOS X',
        'Environment :: Console',
        'Intended Audience :: Developers',
        'Intended Audience :: System Administrators',
        'Natural Language :: English',
        'Operating System :: MacOS',
        'Operating System :: POSIX :: Linux',
        'Topic :: Software Development :: Version Control :: Git',
        'Topic :: System :: Software Distribution',
    ],
    packages=find_packages(exclude=('tests',)),
    include_package_data=True,
    entry_points={
        'console_scripts': [
            'auto-tag=auto_tag.__main__:main',
        ]
    },
    install_reqs=[
        'gitpython',
        'semantic_version',
        'confuse',
        'six',
    ]
)
